// 	GUIDER_DECODER for DSP pcb version 3
//  Fiber interface with 23-STAGE SCRAM/DSCRAM
// 	Author           	Greg Burley
//  Organization        OCIW
//  Revision Level      3.0
//  Target Device       EPM7256AETC144-5

// Module Declaration
module gcam (DQ, D, BA, CS, A, CLK, RESET, RD, WR, AA3, MR, FTXD, FTXCK,
	FRXD, FRXCK, BWR, BRD, SQCK, SQEN, IRQA, IRQB, REFCLK, SDATA, RDATA, RXEN, 
	RXRDY, RCNT, TX_LD, G0, G1, G2, G3, G4, G5, G6, G7);
	
inout	[15:0] DQ;				// Bidirectional backplane data bus
inout	[15:0] D;				// Bidirectional DSP data bus
output	[2:0] BA;				// Buffered backplane address
output	[8:0] CS;				// chip selects for peripherals
input	[7:0] A;				// address bus
input	CLK, RESET;				// DSP clock + global reset
input	RD, WR;					// DSP read + write
input	AA3;					// DSP external peripheral
output	MR;						// reset trigger
output	FTXD, FTXCK;			// optical fiber transmit signals
input	FRXD, FRXCK;			// optical fiber receive signals
output	BWR, BRD;				// buffered write + read signals
output	SQCK, SQEN;				// SEQ register clock + enable
output	IRQA, IRQB;				// DSP interupt lines
output	REFCLK;					// CDR chip reference clock
output	SDATA, RDATA;			// unscrambled send + receive data
output	G0, G1, G2, G3;			// spare pins
output  G4, G5, G6, G7;			// spare pins
output	RXEN;					// RX register enable
output  RXRDY;					// RX ready signal
output	TX_LD;
output	[4:0] RCNT;

wire OEa, OEb;					// bi-directional bus output enables
wire [22:0] SCRAM, DSCRAM;
wire [17:0] TSHIFT, TSHIFT_d;
wire [15:0] RSHIFT, RXREG, DMUX;
wire [4:0] TCNT, TCNT_eq;
wire [4:0] RCNT, RCNT_eq;
wire [5:0] INT, INT_eq;
wire [2:0] CNT;
wire TX_LD, RX_LD;
wire RDATA, RDATA_d;
wire DRXD, DRCNT;
wire RXRDY;
wire SCRAM_in;
wire INT_cout, INT_cnt_en, INT_sclr;


//	DQBUS BRIDGES TO BACKPLANE -- DBUS BRIDGES TO DSP
//  [D->DQ for SEQREG and DACs, DQ->D for ADCs]
//  A = (COND) ? TRUE: FALSE;
assign DQ = (OEb) ? D[15:0] : 16'bz;
assign DMUX = (RXEN) ? DQ : RXREG;
assign D = (OEa) ? DMUX : 16'bz;
assign OEa = (~RD & ~AA3);
assign OEb = (~WR & ~AA3 & (A[7:0]>='H88) );


// 	CLOCK CIRCUIT + TXCLK AND ADCLK OUTPUT
lpm_counter	 cnt(.clock(CLK),.q(CNT));
	defparam cnt.lpm_width=3;
	defparam cnt.lpm_direction="UP";

assign FTXCK = CLK;
assign REFCLK = CNT[2];	//	CLK/8

//	SCRAMBLER + DESCRAMBLER DESCRIPTION -- 23-STAGE
//	SCRAM.shiftin = SDATA $ SCRAM.q[4] $ SCRAM.q[22];
lpm_shiftreg scram(.clock(CLK), .q(SCRAM), .shiftin (SCRAM_in));
	defparam scram.lpm_width=23;
	defparam scram.lpm_direction="LEFT";

assign SCRAM_in = (SDATA ^ SCRAM[4] ^ SCRAM[22]);
assign FTXD = SCRAM[22];

lpm_shiftreg dscram(.clock(FRXCK), .q(DSCRAM), .shiftin(FRXD));
	defparam dscram.lpm_width=23;
	defparam dscram.lpm_direction="LEFT";

assign RDATA_d = (FRXD ^ DSCRAM[4] ^ DSCRAM[22]);
dff	u1(.clk(FRXCK), .d(RDATA_d), .q(RDATA));

// 	TX SHIFT REGISTER AT ADDRESS $FF89 USING /SS1 
lpm_shiftreg tshift(.clock(CLK), .data(TSHIFT_d), .q(TSHIFT), .load(TX_LD),
	.shiftin (~RESET), .shiftout(SDATA));
	defparam tshift.lpm_width=18;
	defparam tshift.lpm_direction="RIGHT";

assign TSHIFT_d[17:0] = {~RESET, D[15:0], 1'b1};
dff	u2(.clk(CLK), .d(~WR & ~AA3 & (A[7:0]=='H85)), 
	.q(TX_LD), .clrn(~WR));

// 	TX FRAME_SYNC/BUSY GENERATOR CIRCUIT -- IRQA IS TX DONE
lpm_counter	tcnt(.clock(CLK), .q(TCNT), .sset(TX_LD), .eq(TCNT_eq), 
	.cnt_en(~TCNT_eq[0]));
	defparam tcnt.lpm_width=5;
	defparam tcnt.lpm_direction="DOWN";
	defparam tcnt.lpm_svalue=18;

assign IRQA = ~TCNT_eq[0] | ~RESET;		// IRQA = VCC during reset 

// 	XREG LOADS FROM RX SHIFT REG -- IRQB IS RX READY
lpm_shiftreg rshift(.clock(FRXCK), .q(RSHIFT), .shiftin(RDATA));
	defparam rshift.lpm_width=16;
	defparam rshift.lpm_direction="RIGHT";

lpm_counter	rcnt(.clock(FRXCK), .q(RCNT), .eq(RCNT_eq), .cnt_en(~RCNT_eq[0]),
	.sset(RDATA & RCNT_eq[0]) );
	defparam rcnt.lpm_width=5;
	defparam rcnt.lpm_direction="DOWN";
	defparam rcnt.lpm_svalue=17;

lpm_ff u3(.clock(FRXCK), .data(RSHIFT[15:0]), .q(RXREG[15:0]), .enable(RCNT_eq[1]));
	defparam u3.lpm_width=16;

dffe u4(.clk(FRXCK), .d(IRQB), .q(RXRDY), .ena(DRCNT), .clrn(RXEN));

assign IRQB = ~RCNT_eq[0] & RESET; 		// IRQB = GND during reset 
assign RXEN = ~(~AA3 & (A[7:0]=='H86));


// 	GRAB + DECODE RESET SIGNAL FROM RX SHIFT REG
dff	u5(.clk(FRXCK), .d(RDATA), .q(DRXD));
dff	u6(.clk(FRXCK), .d(RCNT_eq[2]), .q(DRCNT));

lpm_counter	int (.clock(FRXCK), .q(INT), .eq(INT_eq), .cout(INT_cout),
	.clk_en(DRCNT), .cnt_en(INT_cnt_en), .sclr(INT_sclr), .aclr(~RESET));
	defparam int.lpm_width=6;
	defparam int.lpm_direction="UP";

assign INT_cnt_en = (RDATA & ~DRXD & ~INT_cout);
assign INT_sclr = (DRCNT & (~RDATA | DRXD));

assign MR = (INT_cout) ? ~INT_cout : 1'bz;


//	SEQUENCE REGISTER AND BACKPLANE BUFFER DETAILS --  BRW L=RD H=WR
// 	SEQREG ADDRESS IS $FFFF80 USING /AA3
assign SQCK = WR;
assign SQEN = ~(~AA3 & (A[7:0]=='H80));
assign BWR = WR;
assign BRD = RD;
assign BA[2:0] = A[2:0];


// INITIALIZATION SIGNAL FOR DSP PIN

//assign IRQA = 1;

//	CHIP SELECT		DECODED ADDRESS			DEVICE
//	===========		==================		=======
//	/CS0			$FFFF80					SEQ_REG	
//	/CS1			$FFFF81					ADC_A
//	/CS2			$FFFF82					ADC_B
//	/CS3			$FFFF88 -- $FFFF8F		DAC_1 [BIAS0..7]
//	/CS4			$FFFF90 -- $FFFF97		DAC_2 [V0..8]
//	/CS5			$FFFF98 -- $FFFF9F		DAC_3 [V9..15]

//	 				$FFFF85					TXREG
//					$FFFF86					RXREG

assign CS[0] = ~((A[7:0] == 'H80) & ~AA3 & ~WR);
assign CS[1] = ~((A[7:0] == 'H81) & ~AA3 & ~RD);
assign CS[2] = ~((A[7:0] == 'H82) & ~AA3 & ~RD);

assign CS[3] = ~((A[7:0] >= 'H88) & (A[7:0] <= 'H8F) & ~AA3);
assign CS[4] = ~((A[7:0] >= 'H90) & (A[7:0] <= 'H97) & ~AA3);
assign CS[5] = ~((A[7:0] >= 'H98) & (A[7:0] <= 'H9F) & ~AA3);

assign CS[6] = ~((A[7:0] == 'H83) & ~AA3 & ~RD);
assign CS[7] = ~((A[7:0] == 'H84) & ~AA3 & ~RD);
assign CS[8] = ~((A[7:0] >= 'HA0) & (A[7:0] <= 'HA7) & ~AA3);

//	EXTRA PINS FOR FUTURE USE (?)
assign G0 = 1'bz;
assign G1 = 1'bz;
assign G2 = 1'bz;
assign G3 = 1'bz;
assign G4 = 1'bz;
assign G5 = 1'bz;
assign G6 = 1'bz;
assign G7 = 1'bz;

endmodule



